#!/bin/bash

##### About #####
echo ""
echo "##### ABOUT #####"
echo "-----------------"
echo "Name: Reads Quality Control Pipeline"
echo "Author: Nicolas Fernandez"
echo "Affiliation: IRD_U233_TransVIHMI"
echo "Aim: Reads Quality Control and Cleaning"
echo "Date: 2021.04.30"
echo "Run: snakemake --use-conda -s reads_quality_control_pipeline.smk --cores"
echo "Latest modification: 2021.10.14"
echo "Todo: na"
echo "________________________________________________________________________"

##### Hardware check #####
echo ""
echo "##### HARDWARE #####"
echo "--------------------"
physicalcpu=$(sysctl -n hw.physicalcpu)   # Get physical cpu
echo "Physical CPU: ${physicalcpu}"       # Print physical cpu
logicalcpu=$(sysctl -n hw.logicalcpu)     # Get logical cpu
echo "Logical CPU: ${logicalcpu}"         # Print logical cpu
hwmemsize=$(sysctl -n hw.memsize)         # Get memory size
ramsize=$(expr $hwmemsize / $((1024**3))) # 1024**3 = GB
echo "System Memory: ${ramsize} GB"       # Print RAM size
timestamp=$(date +'%Y-%m-%d_%H-%M')
echo "Date: ${timestamp}"
echo "________________________________________________________________________"

##### Working directory #####
echo ""
echo "##### WORKING DIRECTORY #####"
echo "-----------------------------"
workdir=${0%/*} # for MacOS users, running script with double-click on "RQCP.sh"
#workdir="." # for Linux users (todo), running script with terminal command "bash ./RQCP.sh"
echo "CWD: ${workdir}"
echo "________________________________________________________________________"

#### Get project name 
echo ""
echo "#### GET PROJECT NAME ####"
echo "--------------------------"
read -p "Please enter a project name: " project
echo "Project name: ${project}"
echo "________________________________________________________________________"

###### Create links for raw samples #####
echo ""
echo "##### CREATE RAW READ LINKS #####"
echo "---------------------------------"
mkdir ${workdir}/results/ 2> /dev/null
mkdir ${workdir}/results/reads/ 2> /dev/null
mkdir ${workdir}/results/reads/raw_links/ 2> /dev/null
# Create Symbolic links for raw reads
for fastq in ${workdir}/resources/reads/*.fastq.gz ; do
    ln -s $fastq ${workdir}/results/reads/raw_links/$(basename $fastq) 2> /dev/null ;
done
echo "Silence mode"
echo "________________________________________________________________________"

###### Rename links #####
# comment first line if you want do keep barcode-ID in name
echo ""
echo "##### RENAME FASTQ FILES #####"
echo "------------------------------"
rename -v 's/_S\d+_/_/' ${workdir}/results/reads/raw_links/*.fastq.gz 2> /dev/null                # Remove barcode-ID like {_S001_}
rename -v 's/_L\d+_/_/' ${workdir}/results/reads/raw_links/*.fastq.gz 2> /dev/null                # Remove line-ID ID like {_L001_}
rename -v 's/_\d+.fastq.gz/.fastq.gz/' ${workdir}/results/reads/raw_links/*.fastq.gz 2> /dev/null # Remove end-name ID like {_001}.fastq.gz
echo "Silence mode"
echo "________________________________________________________________________"

###### Create dir for QC tools #####
mkdir ${workdir}/results/reads/cutadapt_removed/ 2> /dev/null
mkdir ${workdir}/results/reads/sickle-trim_paired-end_filtered/ 2> /dev/null
mkdir ${workdir}/results/reads/fastq-join_single-end/ 2> /dev/null
mkdir ${workdir}/results/reads/all_merged_single-end/ 2> /dev/null

###### Extract bwa indexes for small genomes #####
echo ""
echo "##### BOWTIE2 INDEXES EXTRACTION #####"
echo "---------------------------------"
tar --keep-old-files -xzvf ${workdir}/resources/databases/bowtie2_small_genomes.tar.gz -C ${workdir}/resources/databases/ # 2> /dev/null
#tar --keep-old-files -xzvf ${workdir}/resources/databases/bowtie2_large_genomes.tar.gz -C ${workdir}/resources/databases/ # 2> /dev/null
echo "________________________________________________________________________"

###### Call snakemake pipeline #####
echo ""
echo "##### SNAKEMAKE PIPELIE #####"
echo "-----------------------------"
echo ""
echo "Unlocking working directory:"
snakemake --directory ${workdir} --use-conda --keep-going --rerun-incomplete -s ${workdir}/workflow/rules/reads_quality_control_pipeline.smk --cores --unlock # unlock first, if previous error
echo ""
echo "Dry run:"
snakemake --directory ${workdir} --use-conda --keep-going --rerun-incomplete -s ${workdir}/workflow/rules/reads_quality_control_pipeline.smk --cores --dryrun # then dry run, check error like missing file
echo ""
echo "Let's go!"
snakemake --directory ${workdir} --use-conda --keep-going --rerun-incomplete -s ${workdir}/workflow/rules/reads_quality_control_pipeline.smk --cores          # at last, real run 
echo "________________________________________________________________________"

###### Create usefull graphs and summary ######
echo ""
echo "##### SNAKEMAKE PIPELINE GRAPHS ######"
echo "--------------------------------------"
mkdir ${workdir}/results/graphs/ 2> /dev/null
snakemake --directory ${workdir} --use-conda -s ${workdir}/workflow/rules/reads_quality_control_pipeline.smk --dag       | dot -Tpdf > ${workdir}/results/graphs/DAGraph.pdf
snakemake --directory ${workdir} --use-conda -s ${workdir}/workflow/rules/reads_quality_control_pipeline.smk --rulegraph | dot -Tpdf > ${workdir}/results/graphs/RuleGraph.pdf
snakemake --directory ${workdir} --use-conda -s ${workdir}/workflow/rules/reads_quality_control_pipeline.smk --filegraph | dot -Tpdf > ${workdir}/results/graphs/FileGraph.pdf
snakemake --directory ${workdir} --use-conda -s ${workdir}/workflow/rules/reads_quality_control_pipeline.smk --summary   >             ${workdir}/results/graphs/Summary.txt
echo "________________________________________________________________________"

###### Rename results directory with timestamp and project name ######
echo ""
echo "##### SCRIPT END ######"
echo "-----------------------"
mv ${workdir}/results/ ${workdir}/results_${timestamp}_${project}/
